from enum import Enum


class RelationshipType(Enum):
    IS_A = "be"
    DO = "do"
    RELATED_TO = "be related to"
    FORM_OF = "be form of"
    PART_OF = "be part of"
    HAS_A = "have"
    USED_FOR = "use for"
    CAPABLE_OF = "capable of"
    AT_LOCATION = "be at location"
    CAUSES = "cause"
    HAS_SUBEVENT = "have subevent"
    HAS_FIRST_SUBEVENT = "have first subevent"
    HAS_LAST_SUBEVENT = "have last subevent"
    HAS_PREREQUISITE = "have prerequisite"
    HAS_PROPERTY = "have property"
    MOTIVATED_BY_GOAL = "motivate by goal"
    OBSTRUCTED_BY = "obstruct by"
    DESIRES = "desire"
    CREATED_BY = "create by"
    SYNONYM = "synonym"
    ANTONYM = "antonym"
    DISTINCT_FROM = "be distinct from"
    DERIVED_FROM = "be derived from"
    SYMBOL_OF = "be symbol of"
    DEFINED_AS = "be defined as"
    MANNER_OF = "be manner of"
    LOCATED_NEAR = "be located near"
    HAS_CONTEXT = "have context"
    SIMILAR_TO = "be similar to"
    ETYMOLOGICALLY_RELATED_TO = "be etymologically related to"
    ETYMOLOGICALLY_DERIVED_FROM = "be etymologically derived from"
    CAUSES_DESIRE = "cause desire"
    MADE_OF = "be made of"
    RECEIVES_ACTION = "receive action"
